@@ -14,7 +14,7 @@
 #include "vector.h"
 #include "config.h"
 
-void dlog (int sink, int prio, char * fmt, ...)
+void dlog (int sink, int prio, const char * fmt, ...)
 {
 	va_list ap;
 	int thres;
@@ -29,17 +29,16 @@ void dlog (int sink, int prio, char * fmt, ...)
 			struct tm *tb = localtime(&t);
 			char buff[16];
 			
-			strftime(buff, 16, "%b %d %H:%M:%S", tb); 
+			strftime(buff, sizeof(buff), "%b %d %H:%M:%S", tb);
+			buff[sizeof(buff)-1] = '\0';
 
 			fprintf(stdout, "%s | ", buff);
 			vfprintf(stdout, fmt, ap);
-			fprintf(stdout, "\n");
 		}
 		else
 			log_safe(prio + 3, fmt, ap);
 #else
 		vfprintf(stdout, fmt, ap);
-		fprintf(stdout, "\n");
 #endif
 	}
 	va_end(ap);
