@@ -200,6 +200,32 @@ def_no_path_retry_handler(vector strvec)
 	return 0;
 }
 
+static int
+def_pg_timeout_handler(vector strvec)
+{
+	int pg_timeout;
+	char * buff;
+
+	buff = set_value(strvec);
+
+	if (!buff)
+		return 1;
+
+	if (strlen(buff) == 4 && !strcmp(buff, "none"))
+		conf->pg_timeout = -PGTIMEOUT_NONE;
+	else if (sscanf(buff, "%d", &pg_timeout) == 1 && pg_timeout >= 0) {
+		if (pg_timeout == 0)
+			conf->pg_timeout = -PGTIMEOUT_NONE;
+		else
+			conf->pg_timeout = pg_timeout;
+	}
+	else
+		conf->pg_timeout = PGTIMEOUT_UNDEF;
+
+	FREE(buff);
+	return 0;
+}
+
 static int
 names_handler(vector strvec)
 {
@@ -237,6 +263,19 @@ blacklist_handler(vector strvec)
 	return 0;
 }
 
+static int
+blacklist_exceptions_handler(vector strvec)
+{
+	conf->elist_devnode = vector_alloc();
+	conf->elist_wwid = vector_alloc();
+	conf->elist_device = vector_alloc();
+
+	if (!conf->elist_devnode || !conf->elist_wwid || !conf->elist_device)
+		return 1;
+
+	return 0;
+}
+
 static int
 ble_devnode_handler(vector strvec)
 {
@@ -247,7 +286,20 @@ ble_devnode_handler(vector strvec)
 	if (!buff)
 		return 1;
 
-	return store_ble(conf->blist_devnode, buff);
+	return store_ble(conf->blist_devnode, buff, ORIGIN_CONFIG);
+}
+
+static int
+ble_except_devnode_handler(vector strvec)
+{
+	char * buff;
+
+	buff = set_value(strvec);
+
+	if (!buff)
+		return 1;
+
+	return store_ble(conf->elist_devnode, buff, ORIGIN_CONFIG);
 }
 
 static int
@@ -260,7 +312,20 @@ ble_wwid_handler(vector strvec)
 	if (!buff)
 		return 1;
 
-	return store_ble(conf->blist_wwid, buff);
+	return store_ble(conf->blist_wwid, buff, ORIGIN_CONFIG);
+}
+
+static int
+ble_except_wwid_handler(vector strvec)
+{
+        char * buff;
+
+        buff = set_value(strvec);
+
+        if (!buff)
+                return 1;
+
+	return store_ble(conf->elist_wwid, buff, ORIGIN_CONFIG);
 }
 
 static int
@@ -269,6 +334,12 @@ ble_device_handler(vector strvec)
 	return alloc_ble_device(conf->blist_device);
 }
 
+static int
+ble_except_device_handler(vector strvec)
+{
+	return alloc_ble_device(conf->elist_device);
+}
+
 static int
 ble_vendor_handler(vector strvec)
 {
@@ -279,7 +350,20 @@ ble_vendor_handler(vector strvec)
 	if (!buff)
 		return 1;
 
-	return set_ble_device(conf->blist_device, buff, NULL);
+	return set_ble_device(conf->blist_device, buff, NULL, ORIGIN_CONFIG);
+}
+
+static int
+ble_except_vendor_handler(vector strvec)
+{
+	char * buff;
+
+	buff = set_value(strvec);
+
+	if (!buff)
+		return 1;
+
+	return set_ble_device(conf->elist_device, buff, NULL, ORIGIN_CONFIG);
 }
 
 static int
@@ -292,7 +376,20 @@ ble_product_handler(vector strvec)
 	if (!buff)
 		return 1;
 
-	return set_ble_device(conf->blist_device, NULL, buff);
+	return set_ble_device(conf->blist_device, NULL, buff, ORIGIN_CONFIG);
+}
+
+static int
+ble_except_product_handler(vector strvec)
+{
+	char * buff;
+
+	buff = set_value(strvec);
+
+	if (!buff)
+		return 1;
+
+	return set_ble_device(conf->elist_device, NULL, buff, ORIGIN_CONFIG);
 }
 
 /*
@@ -585,6 +682,36 @@ hw_minio_handler(vector strvec)
 	return 0;
 }
 
+static int
+hw_pg_timeout_handler(vector strvec)
+{
+	int pg_timeout;
+	struct hwentry *hwe = VECTOR_LAST_SLOT(conf->hwtable);
+	char *buff;
+
+	if (!hwe)
+		return 1;
+
+	buff = set_value(strvec);
+
+	if (!buff)
+		return 1;
+
+	if (strlen(buff) == 4 && !strcmp(buff, "none"))
+		hwe->pg_timeout = -PGTIMEOUT_NONE;
+	else if (sscanf(buff, "%d", &pg_timeout) == 1 && pg_timeout >= 0) {
+		if (pg_timeout == 0)
+			hwe->pg_timeout = -PGTIMEOUT_NONE;
+		else
+			hwe->pg_timeout = pg_timeout;
+	}
+	else
+		hwe->pg_timeout = PGTIMEOUT_UNDEF;
+
+	FREE(buff);
+	return 0;
+}
+
 /*
  * multipaths block handlers
  */
@@ -777,6 +904,35 @@ mp_minio_handler(vector strvec)
 	return 0;
 }
 
+static int
+mp_pg_timeout_handler(vector strvec)
+{
+	int pg_timeout;
+	struct mpentry *mpe = VECTOR_LAST_SLOT(conf->mptable);
+	char *buff;
+
+	if (!mpe)
+		return 1;
+
+	buff = set_value(strvec);
+
+	if (!buff)
+		return 1;
+	if (strlen(buff) == 4 && !strcmp(buff, "none"))
+		mpe->pg_timeout = -PGTIMEOUT_NONE;
+	else if (sscanf(buff, "%d", &pg_timeout) == 1 && pg_timeout >= 0) {
+		if (pg_timeout == 0)
+			mpe->pg_timeout = -PGTIMEOUT_NONE;
+		else
+			mpe->pg_timeout = pg_timeout;
+	}
+	else
+		mpe->pg_timeout = PGTIMEOUT_UNDEF;
+
+	FREE(buff);
+	return 0;
+}
+
 /*
  * config file keywords printing
  */
@@ -895,6 +1051,22 @@ snprint_mp_rr_min_io (char * buff, int len, void * data)
 	return snprintf(buff, len, "%u", mpe->minio);
 }
 
+static int
+snprint_mp_pg_timeout (char * buff, int len, void * data)
+{
+	struct mpentry * mpe = (struct mpentry *)data;
+
+	switch (mpe->pg_timeout) {
+	case PGTIMEOUT_UNDEF:
+		break;
+	case -PGTIMEOUT_NONE:
+		return snprintf(buff, len, "none");
+	default:
+		return snprintf(buff, len, "%i", mpe->pg_timeout);
+	}
+	return 0;
+}
+
 static int
 snprint_hw_vendor (char * buff, int len, void * data)
 {
@@ -1096,6 +1268,27 @@ snprint_hw_rr_min_io (char * buff, int len, void * data)
 	return snprintf(buff, len, "%u", hwe->minio);
 }
 
+static int
+snprint_hw_pg_timeout (char * buff, int len, void * data)
+{
+	struct hwentry * hwe = (struct hwentry *)data;
+
+	if (!hwe->pg_timeout)
+		return 0;
+	if (hwe->pg_timeout == conf->pg_timeout)
+		return 0;
+
+	switch (hwe->pg_timeout) {
+	case PGTIMEOUT_UNDEF:
+		break;
+	case -PGTIMEOUT_NONE:
+		return snprintf(buff, len, "none");
+	default:
+		return snprintf(buff, len, "%i", hwe->pg_timeout);
+	}
+	return 0;
+}
+
 static int
 snprint_hw_path_checker (char * buff, int len, void * data)
 {
@@ -1267,6 +1460,23 @@ snprint_def_no_path_retry (char * buff, int len, void * data)
 	return 0;
 }
 
+static int
+snprint_def_pg_timeout (char * buff, int len, void * data)
+{
+	if (conf->pg_timeout == DEFAULT_PGTIMEOUT)
+		return 0;
+
+	switch (conf->pg_timeout) {
+	case PGTIMEOUT_UNDEF:
+		break;
+	case -PGTIMEOUT_NONE:
+		return snprintf(buff, len, "none");
+	default:
+		return snprintf(buff, len, "%i", conf->pg_timeout);
+	}
+	return 0;
+}
+
 static int
 snprint_def_user_friendly_names (char * buff, int len, void * data)
 {
@@ -1320,6 +1530,7 @@ init_keywords(void)
 	install_keyword("rr_min_io", &def_minio_handler, &snprint_def_rr_min_io);
 	install_keyword("rr_weight", &def_weight_handler, &snprint_def_rr_weight);
 	install_keyword("no_path_retry", &def_no_path_retry_handler, &snprint_def_no_path_retry);
+	install_keyword("pg_timeout", &def_pg_timeout_handler, &snprint_def_pg_timeout);
 	install_keyword("user_friendly_names", &names_handler, &snprint_def_user_friendly_names);
 	__deprecated install_keyword("default_selector", &def_selector_handler, NULL);
 	__deprecated install_keyword("default_path_grouping_policy", &def_pgpolicy_handler, NULL);
@@ -1336,6 +1547,14 @@ init_keywords(void)
 	install_keyword("vendor", &ble_vendor_handler, &snprint_bled_vendor);
 	install_keyword("product", &ble_product_handler, &snprint_bled_product);
 	install_sublevel_end();
+	install_keyword_root("blacklist_exceptions", &blacklist_exceptions_handler);
+	install_keyword("devnode", &ble_except_devnode_handler, &snprint_ble_simple);
+	install_keyword("wwid", &ble_except_wwid_handler, &snprint_ble_simple);
+	install_keyword("device", &ble_except_device_handler, NULL);
+	install_sublevel();
+	install_keyword("vendor", &ble_except_vendor_handler, &snprint_bled_vendor);
+	install_keyword("product", &ble_except_product_handler, &snprint_bled_product);
+	install_sublevel_end();
 
 #if 0
 	__deprecated install_keyword_root("devnode_blacklist", &blacklist_handler);
@@ -1365,6 +1584,7 @@ init_keywords(void)
 	install_keyword("rr_weight", &hw_weight_handler, &snprint_hw_rr_weight);
 	install_keyword("no_path_retry", &hw_no_path_retry_handler, &snprint_hw_no_path_retry);
 	install_keyword("rr_min_io", &hw_minio_handler, &snprint_hw_rr_min_io);
+	install_keyword("pg_timeout", &hw_pg_timeout_handler, &snprint_hw_pg_timeout);
 	install_sublevel_end();
 
 	install_keyword_root("multipaths", &multipaths_handler);
@@ -1378,5 +1598,6 @@ init_keywords(void)
 	install_keyword("rr_weight", &mp_weight_handler, &snprint_mp_rr_weight);
 	install_keyword("no_path_retry", &mp_no_path_retry_handler, &snprint_mp_no_path_retry);
 	install_keyword("rr_min_io", &mp_minio_handler, &snprint_mp_rr_min_io);
+	install_keyword("pg_timeout", &mp_pg_timeout_handler, &snprint_mp_pg_timeout);
 	install_sublevel_end();
 }
