@@ -5,6 +5,8 @@
 #include <string.h>
 #include <libdevmapper.h>
 #include <stdarg.h>
+#include <sys/stat.h>
+#include <dirent.h>
 
 #include <checkers.h>
 
@@ -13,11 +15,12 @@
 #include "structs_vec.h"
 #include "print.h"
 #include "dmparser.h"
-#include "configure.h"
 #include "config.h"
+#include "configure.h"
 #include "pgpolicies.h"
 #include "defaults.h"
 #include "parser.h"
+#include "blacklist.h"
 
 #define MAX(x,y) (x > y) ? x : y
 #define TAIL     (line + len - 1 - c)
@@ -52,16 +55,26 @@ snprint_uint (char * buff, size_t len, unsigned int val)
 static int
 snprint_size (char * buff, size_t len, unsigned long long size)
 {
-	if (size < (1 << 11))
-		return snprintf(buff, len, "%llu kB", size >> 1);
-	else if (size < (1 << 21))
-		return snprintf(buff, len, "%llu MB", size >> 11);
-	else if (size < (1 << 31))
-		return snprintf(buff, len, "%llu GB", size >> 21);
+	float s = (float)(size >> 1); /* start with KB */
+	char fmt[6] = {};
+	char units[] = {'K','M','G','T','P'};
+	char *u = units;
+	
+	while (s >= 1024 && *u != 'P') {
+		s = s / 1024;
+		u++;
+	}
+	if (s < 10)
+		snprintf(fmt, 6, "%%.1f%c", *u);
 	else
-		return snprintf(buff, len, "%llu TB", size >> 31);
+		snprintf(fmt, 6, "%%.0f%c", *u);
+
+	return snprintf(buff, len, fmt, s);
 }
 
+/*
+ * multipath info printing functions
+ */
 static int
 snprint_name (char * buff, size_t len, struct multipath * mpp)
 {
@@ -211,6 +224,10 @@ static int
 snprint_action (char * buff, size_t len, struct multipath * mpp)
 {
 	switch (mpp->action) {
+	case ACT_REJECT:
+		return snprint_str(buff, len, ACT_REJECT_STR);
+	case ACT_RENAME:
+		return snprint_str(buff, len, ACT_RENAME_STR);
 	case ACT_RELOAD:
 		return snprint_str(buff, len, ACT_RELOAD_STR);
 	case ACT_CREATE:
@@ -222,6 +239,9 @@ snprint_action (char * buff, size_t len, struct multipath * mpp)
 	}
 }
 
+/*
+ * path info printing functions
+ */
 static int
 snprint_path_uuid (char * buff, size_t len, struct path * pp)
 {
@@ -292,8 +312,8 @@ snprint_dm_path_state (char * buff, size_t len, struct path * pp)
 static int
 snprint_vpr (char * buff, size_t len, struct path * pp)
 {
-	return snprintf(buff, len, "%s/%s/%s",
-		        pp->vendor_id, pp->product_id, pp->rev);
+	return snprintf(buff, len, "%s,%s",
+		        pp->vendor_id, pp->product_id);
 }
 
 static int
@@ -496,7 +516,7 @@ snprint_multipath (char * line, int len, char * format,
 	char * f = format; /* format string cursor */
 	int fwd;
 	struct multipath_data * data;
-	char buff[MAX_FIELD_LEN];
+	char buff[MAX_FIELD_LEN] = {};
 
 	do {
 		if (!TAIL)
@@ -515,6 +535,7 @@ snprint_multipath (char * line, int len, char * format,
 		data->snprint(buff, MAX_FIELD_LEN, mpp);
 		PRINT(c, TAIL, buff);
 		PAD(data->width);
+		buff[0] = '\0';
 	} while (*f++);
 
 	line[c - line - 1] = '\n';
@@ -631,7 +652,7 @@ snprint_pathgroup (char * line, int len, char * format,
 extern void
 print_multipath_topology (struct multipath * mpp, int verbosity)
 {
-	char buff[MAX_LINE_LEN * MAX_LINES];
+	char buff[MAX_LINE_LEN * MAX_LINES] = {};
 
 	snprint_multipath_topology(&buff[0], MAX_LINE_LEN * MAX_LINES,
 				   mpp, verbosity);
@@ -662,7 +683,10 @@ snprint_multipath_topology (char * buff, int len, struct multipath * mpp,
 	c += sprintf(c, "%%n");
 	
 	if (strncmp(mpp->alias, mpp->wwid, WWID_SIZE))
-		c += sprintf(c, " (%%w)");
+		c += sprintf(c, " (%%w) ");
+
+	c += sprintf(c, "%%d ");
+	c += snprint_vpr(c, 24, first_path(mpp));
 
 	fwd += snprint_multipath(buff + fwd, len - fwd, style, mpp);
 	if (fwd > len)
@@ -833,6 +857,110 @@ snprint_defaults (char * buff, int len)
 	
 }
 
+static int
+snprint_blacklist_group (char *buff, int len, int *fwd, vector *vec)
+{
+	int threshold = MAX_LINE_LEN;
+	struct blentry * ble;
+	int pos;
+	int i;
+
+	pos = *fwd;
+	if (!VECTOR_SIZE(*vec)) {
+		if ((len - pos - threshold) <= 0)
+			return 0;
+		pos += snprintf(buff + pos, len - pos, "        <empty>\n");
+	} else vector_foreach_slot (*vec, ble, i) {
+		if ((len - pos - threshold) <= 0)
+			return 0;
+		if (ble->origin == ORIGIN_CONFIG)
+			pos += snprintf(buff + pos, len - pos, "        (config file rule) ");
+		else if (ble->origin == ORIGIN_DEFAULT)
+			pos += snprintf(buff + pos, len - pos, "        (default rule)     ");
+		pos += snprintf(buff + pos, len - pos, "%s\n", ble->str);
+	}
+
+	*fwd = pos;
+	return pos;
+}
+
+static int
+snprint_blacklist_devgroup (char *buff, int len, int *fwd, vector *vec)
+{
+	int threshold = MAX_LINE_LEN;
+	struct blentry_device * bled;
+	int pos;
+	int i;
+
+	pos = *fwd;
+	if (!VECTOR_SIZE(*vec)) {
+		if ((len - pos - threshold) <= 0)
+			return 0;
+		pos += snprintf(buff + pos, len - pos, "        <empty>\n");
+	} else vector_foreach_slot (*vec, bled, i) {
+		if ((len - pos - threshold) <= 0)
+			return 0;
+		if (bled->origin == ORIGIN_CONFIG)
+			pos += snprintf(buff + pos, len - pos, "        (config file rule) ");
+		else if (bled->origin == ORIGIN_DEFAULT)
+			pos += snprintf(buff + pos, len - pos, "        (default rule)     ");
+		pos += snprintf(buff + pos, len - pos, "%s:%s\n", bled->vendor, bled->product);
+	}
+
+	*fwd = pos;
+	return pos;
+}
+
+extern int
+snprint_blacklist_report (char * buff, int len)
+{
+	int threshold = MAX_LINE_LEN;
+	int fwd = 0;
+
+	if ((len - fwd - threshold) <= 0)
+		return len;
+	fwd += snprintf(buff + fwd, len - fwd, "device node rules:\n"
+			                       "- blacklist:\n");
+	if (!snprint_blacklist_group(buff, len, &fwd, &conf->blist_devnode))
+		return len;
+
+	if ((len - fwd - threshold) <= 0)
+		return len;
+	fwd += snprintf(buff + fwd, len - fwd, "- exceptions:\n");
+	if (snprint_blacklist_group(buff, len, &fwd, &conf->elist_devnode) == 0)
+		return len;
+
+	if ((len - fwd - threshold) <= 0)
+		return len;
+	fwd += snprintf(buff + fwd, len - fwd, "wwid rules:\n"
+			                       "- blacklist:\n");
+	if (snprint_blacklist_group(buff, len, &fwd, &conf->blist_wwid) == 0)
+		return len;
+
+	if ((len - fwd - threshold) <= 0)
+		return len;
+	fwd += snprintf(buff + fwd, len - fwd, "- exceptions:\n");
+	if (snprint_blacklist_group(buff, len, &fwd, &conf->elist_wwid) == 0)
+		return len;
+
+	if ((len - fwd - threshold) <= 0)
+		return len;
+	fwd += snprintf(buff + fwd, len - fwd, "device rules:\n"
+			                       "- blacklist:\n");
+	if (snprint_blacklist_devgroup(buff, len, &fwd, &conf->blist_device) == 0)
+		return len;
+
+	if ((len - fwd - threshold) <= 0)
+		return len;
+	fwd += snprintf(buff + fwd, len - fwd, "- exceptions:\n");
+	if (snprint_blacklist_devgroup(buff, len, &fwd, &conf->elist_device) == 0)
+		return len;
+
+	if (fwd > len)
+		return len;
+	return fwd;
+}
+
 extern int
 snprint_blacklist (char * buff, int len)
 {
@@ -895,12 +1023,137 @@ snprint_blacklist (char * buff, int len)
 		if (fwd > len)
 			return len;
 	}
-
 	fwd += snprintf(buff + fwd, len - fwd, "}\n");
 	if (fwd > len)
 		return len;
 	return fwd;
-	
+}
+
+extern int
+snprint_blacklist_except (char * buff, int len)
+{
+	int i;
+	struct blentry * ele;
+	struct blentry_device * eled;
+	int fwd = 0;
+	struct keyword *rootkw;
+	struct keyword *kw;
+
+	rootkw = find_keyword(NULL, "blacklist_exceptions");
+	if (!rootkw)
+		return 0;
+
+	fwd += snprintf(buff + fwd, len - fwd, "blacklist_exceptions {\n");
+	if (fwd > len)
+		return len;
+
+	vector_foreach_slot (conf->elist_devnode, ele, i) {
+		kw = find_keyword(rootkw->sub, "devnode");
+		if (!kw)
+			return 0;
+		fwd += snprint_keyword(buff + fwd, len - fwd, "\t%k %v\n",
+				       kw, ele);
+		if (fwd > len)
+			return len;
+	}
+	vector_foreach_slot (conf->elist_wwid, ele, i) {
+		kw = find_keyword(rootkw->sub, "wwid");
+		if (!kw)
+			return 0;
+		fwd += snprint_keyword(buff + fwd, len - fwd, "\t%k %v\n",
+				       kw, ele);
+		if (fwd > len)
+			return len;
+	}
+	rootkw = find_keyword(rootkw->sub, "device");
+	if (!rootkw)
+		return 0;
+
+	vector_foreach_slot (conf->elist_device, eled, i) {
+		fwd += snprintf(buff + fwd, len - fwd, "\tdevice {\n");
+		if (fwd > len)
+			return len;
+		kw = find_keyword(rootkw->sub, "vendor");
+		if (!kw)
+			return 0;
+		fwd += snprint_keyword(buff + fwd, len - fwd, "\t\t%k %v\n",
+				       kw, eled);
+		if (fwd > len)
+			return len;
+		kw = find_keyword(rootkw->sub, "product");
+		if (!kw)
+			return 0;
+		fwd += snprint_keyword(buff + fwd, len - fwd, "\t\t%k %v\n",
+				       kw, eled);
+		if (fwd > len)
+			return len;
+		fwd += snprintf(buff + fwd, len - fwd, "\t}\n");
+		if (fwd > len)
+			return len;
+	}
+	fwd += snprintf(buff + fwd, len - fwd, "}\n");
+	if (fwd > len)
+		return len;
+	return fwd;
+}
+
+extern int
+snprint_devices (char * buff, int len, struct vectors *vecs)
+{
+	DIR *blkdir;
+	struct dirent *blkdev;
+	struct stat statbuf;
+	char devpath[PATH_MAX];
+	char *devptr;
+	int threshold = MAX_LINE_LEN;
+	int fwd = 0;
+	int r;
+
+	struct path * pp;
+
+	if (!(blkdir = opendir("/sys/block")))
+		return 1;
+
+	if ((len - fwd - threshold) <= 0)
+		return len;
+	fwd += snprintf(buff + fwd, len - fwd, "available block devices:\n");
+
+	strcpy(devpath,"/sys/block");
+	devptr = devpath + 10;
+	while ((blkdev = readdir(blkdir)) != NULL) {
+		if ((strcmp(blkdev->d_name,".") == 0) ||
+		    (strcmp(blkdev->d_name,"..") == 0))
+			continue;
+
+		strcat(devptr,blkdev->d_name);
+		if (stat(devptr, &statbuf) < 0)
+			continue;
+
+		if (S_ISDIR(statbuf.st_mode) == 0)
+			continue;
+
+		if ((len - fwd - threshold)  <= 0)
+			return len;
+
+		fwd += snprintf(buff + fwd, len - fwd, "    %s", devpath);
+		pp = find_path_by_dev(vecs->pathvec, devpath);
+		if (!pp) {
+			r = filter_devnode(conf->blist_devnode,
+					   conf->elist_devnode, devpath);
+			if (r > 0)
+				fwd += snprintf(buff + fwd, len - fwd,
+						" (blacklisted)");
+			else if (r < 0)
+				fwd += snprintf(buff + fwd, len - fwd,
+						" (whitelisted)");
+		}
+		fwd += snprintf(buff + fwd, len - fwd, "\n");
+	}
+	closedir(blkdir);
+
+	if (fwd > len)
+		return len;
+	return fwd;
 }
 
 extern int
