@@ -16,6 +16,7 @@
 #include "debug.h"
 #include "structs_vec.h"
 #include "blacklist.h"
+#include "waiter.h"
 
 struct path *
 alloc_path (void)
@@ -30,6 +31,7 @@ alloc_path (void)
 		pp->sg_id.scsi_id = -1;
 		pp->sg_id.lun = -1;
 		pp->fd = -1;
+		pp->priority = PRIO_UNDEF;
 	}
 	return pp;
 }
@@ -115,9 +117,10 @@ alloc_multipath (void)
 
 	mpp = (struct multipath *)MALLOC(sizeof(struct multipath));
 
-	if (mpp)
+	if (mpp) {
 		mpp->bestpg = 1;
-
+		mpp->mpcontext = NULL;
+	}
 	return mpp;
 }
 
@@ -178,6 +181,7 @@ free_multipath (struct multipath * mpp, int free_paths)
 
 	free_pathvec(mpp->paths, free_paths);
 	free_pgvec(mpp->pg, free_paths);
+	FREE_PTR(mpp->mpcontext);
 	FREE(mpp);
 }
 
@@ -365,3 +369,10 @@ pathcount (struct multipath * mpp, int state)
 
 	return count;
 }
+
+struct path *
+first_path (struct multipath * mpp)
+{
+	struct pathgroup * pgp = VECTOR_SLOT(mpp->pg, 0);
+	return VECTOR_SLOT(pgp->paths, 0);
+}
