@@ -13,12 +13,6 @@
 #include <fcntl.h>
 #include <errno.h>
 
-/*
- * libsysfs
- */
-#include <sysfs/libsysfs.h>
-#include <sysfs/dlist.h>
-
 /*
  * libcheckers
  */
@@ -40,6 +34,7 @@
 #include <structs_vec.h>
 #include <dmparser.h>
 #include <devmapper.h>
+#include <sysfs.h>
 #include <dict.h>
 #include <discovery.h>
 #include <debug.h>
@@ -55,136 +50,29 @@
 #include "uxclnt.h"
 #include "cli.h"
 #include "cli_handlers.h"
+#include "lock.h"
+#include "waiter.h"
 
 #define FILE_NAME_SIZE 256
 #define CMDSIZE 160
 
 #define LOG_MSG(a,b) \
-	if (strlen(b)) condlog(a, "%s: %s", pp->dev_t, b);
-
-#ifdef LCKDBG
-#define lock(a) \
-	fprintf(stderr, "%s:%s(%i) lock %p\n", __FILE__, __FUNCTION__, __LINE__, a); \
-	pthread_mutex_lock(a)
-#define unlock(a) \
-	fprintf(stderr, "%s:%s(%i) unlock %p\n", __FILE__, __FUNCTION__, __LINE__, a); \
-	pthread_mutex_unlock(a)
-#define lock_cleanup_pop(a) \
-	fprintf(stderr, "%s:%s(%i) unlock %p\n", __FILE__, __FUNCTION__, __LINE__, a); \
-	pthread_cleanup_pop(1);
-#else
-#define lock(a) pthread_mutex_lock(a)
-#define unlock(a) pthread_mutex_unlock(a)
-#define lock_cleanup_pop(a) pthread_cleanup_pop(1);
-#endif
+	if (strlen(b)) condlog(a, "%s: %s", pp->dev, b);
 
 pthread_cond_t exit_cond = PTHREAD_COND_INITIALIZER;
 pthread_mutex_t exit_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 /*
- * structs
+ * global copy of vecs for use in sig handlers
  */
-struct vectors * gvecs; /* global copy of vecs for use in sig handlers */
-
-static struct event_thread *
-alloc_waiter (void)
-{
-
-	struct event_thread * wp;
-
-	wp = (struct event_thread *)MALLOC(sizeof(struct event_thread));
-
-	return wp;
-}
-
-static void
-free_waiter (void * data)
-{
-	struct event_thread * wp = (struct event_thread *)data;
-
-	/*
-	 * indicate in mpp that the wp is already freed storage
-	 */
-	lock(wp->vecs->lock);
-
-	if (wp->mpp)
-		/*
-		 * be careful, mpp may already be freed -- null if so
-		 */
-		wp->mpp->waiter = NULL;
-	else
-		condlog(3, "free_waiter, mpp freed before wp=%p,", wp);
-
-	unlock(wp->vecs->lock);
-
-	if (wp->dmt)
-		dm_task_destroy(wp->dmt);
-
-	FREE(wp);
-}
-
-static void
-stop_waiter_thread (struct multipath * mpp, struct vectors * vecs)
-{
-	struct event_thread * wp = (struct event_thread *)mpp->waiter;
-	
-	if (!wp) {
-		condlog(3, "%s: no waiter thread", mpp->alias);
-		return;
-	}
-	condlog(2, "%s: stop event checker thread", wp->mapname);
-	pthread_kill((pthread_t)wp->thread, SIGUSR1);
-}
-
-static void
-cleanup_lock (void * data)
-{
-	unlock((pthread_mutex_t *)data);
-}
-
-/*
- * mpp->no_path_retry:
- *   -2 (QUEUE) : queue_if_no_path enabled, never turned off
- *   -1 (FAIL)  : fail_if_no_path
- *    0 (UNDEF) : nothing
- *   >0         : queue_if_no_path enabled, turned off after polling n times
- */
-static void
-update_queue_mode_del_path(struct multipath *mpp)
-{
-	if (--mpp->nr_active == 0 && mpp->no_path_retry > 0) {
-		/*
-		 * Enter retry mode.
-		 * meaning of +1: retry_tick may be decremented in
-		 *                checkerloop before starting retry.
-		 */
-		mpp->stat_queueing_timeouts++;
-		mpp->retry_tick = mpp->no_path_retry * conf->checkint + 1;
-		condlog(1, "%s: Entering recovery mode: max_retries=%d",
-			mpp->alias, mpp->no_path_retry);
-	}
-	condlog(2, "%s: remaining active paths: %d", mpp->alias, mpp->nr_active);
-}
-
-static void
-update_queue_mode_add_path(struct multipath *mpp)
-{
-	if (mpp->nr_active++ == 0 && mpp->no_path_retry > 0) {
-		/* come back to normal mode from retry mode */
-		mpp->retry_tick = 0;
-		dm_queue_if_no_path(mpp->alias, 1);
-		condlog(2, "%s: queue_if_no_path enabled", mpp->alias);
-		condlog(1, "%s: Recovered to normal mode", mpp->alias);
-	}
-	condlog(2, "%s: remaining active paths: %d", mpp->alias, mpp->nr_active);
-}
+struct vectors * gvecs;
 
 static int
 need_switch_pathgroup (struct multipath * mpp, int refresh)
 {
 	struct pathgroup * pgp;
 	struct path * pp;
-	int i, j;
+	unsigned int i, j;
 
 	if (!mpp || mpp->pgfailback == -FAILBACK_MANUAL)
 		return 0;
@@ -219,7 +107,8 @@ coalesce_maps(struct vectors *vecs, vector nmpv)
 {
 	struct multipath * ompp;
 	vector ompv = vecs->mpvec;
-	int i, j;
+	unsigned int i;
+	int j;
 
 	vector_foreach_slot (ompv, ompp, i) {
 		if (!find_mp_by_wwid(nmpv, ompp->wwid)) {
@@ -253,234 +142,12 @@ coalesce_maps(struct vectors *vecs, vector nmpv)
 	return 0;
 }
 
-static int
-update_multipath (struct vectors *vecs, char *mapname)
-{
-	struct multipath *mpp;
-	struct pathgroup  *pgp;
-	struct path *pp;
-	int i, j;
-	int r = 1;
-
-	mpp = find_mp_by_alias(vecs->mpvec, mapname);
-
-	if (!mpp)
-		goto out;
-
-	free_pgvec(mpp->pg, KEEP_PATHS);
-	mpp->pg = NULL;
-
-	if (setup_multipath(vecs, mpp))
-		goto out; /* mpp freed in setup_multipath */
-
-	/*
-	 * compare checkers states with DM states
-	 */
-	vector_foreach_slot (mpp->pg, pgp, i) {
-		vector_foreach_slot (pgp->paths, pp, j) {
-			if (pp->dmstate != PSTATE_FAILED)
-				continue;
-
-			if (pp->state != PATH_DOWN) {
-				int oldstate = pp->state;
-				condlog(2, "%s: mark as failed", pp->dev_t);
-				mpp->stat_path_failures++;
-				pp->state = PATH_DOWN;
-				if (oldstate == PATH_UP ||
-				    oldstate == PATH_GHOST)
-					update_queue_mode_del_path(mpp);
-
-				/*
-				 * if opportune,
-				 * schedule the next check earlier
-				 */
-				if (pp->tick > conf->checkint)
-					pp->tick = conf->checkint;
-			}
-		}
-	}
-	r = 0;
-out:
-	if (r)
-		condlog(0, "failed to update multipath");
-
-	return r;
-}
-
-static sigset_t unblock_signals(void)
-{
-	sigset_t set, old;
-
-	sigemptyset(&set);
-	sigaddset(&set, SIGHUP);
-	sigaddset(&set, SIGUSR1);
-	pthread_sigmask(SIG_UNBLOCK, &set, &old);
-	return old;
-}
-
-/*
- * returns the reschedule delay
- * negative means *stop*
- */
-static int
-waiteventloop (struct event_thread * waiter)
-{
-	sigset_t set;
-	int event_nr;
-	int r;
-
-	if (!waiter->event_nr)
-		waiter->event_nr = dm_geteventnr(waiter->mapname);
-
-	if (!(waiter->dmt = dm_task_create(DM_DEVICE_WAITEVENT))) {
-		condlog(0, "%s: devmap event #%i dm_task_create error",
-				waiter->mapname, waiter->event_nr);
-		return 1;
-	}
-
-	if (!dm_task_set_name(waiter->dmt, waiter->mapname)) {
-		condlog(0, "%s: devmap event #%i dm_task_set_name error",
-				waiter->mapname, waiter->event_nr);
-		dm_task_destroy(waiter->dmt);
-		return 1;
-	}
-
-	if (waiter->event_nr && !dm_task_set_event_nr(waiter->dmt,
-						      waiter->event_nr)) {
-		condlog(0, "%s: devmap event #%i dm_task_set_event_nr error",
-				waiter->mapname, waiter->event_nr);
-		dm_task_destroy(waiter->dmt);
-		return 1;
-	}
-
-	dm_task_no_open_count(waiter->dmt);
-	
-	/* accept wait interruption */
-	set = unblock_signals();
-
-	/* interruption spits messages */
-	dm_shut_log();
-
-	/* wait */
-	r = dm_task_run(waiter->dmt);
-
-	/* wait is over : event or interrupt */
-	pthread_sigmask(SIG_SETMASK, &set, NULL);
-	//dm_restore_log();
-
-	if (!r) /* wait interrupted by signal */
-		return -1;
-
-	dm_task_destroy(waiter->dmt);
-	waiter->dmt = NULL;
-	waiter->event_nr++;
-
-	/*
-	 * upon event ...
-	 */
-	while (1) {
-		condlog(3, "%s: devmap event #%i",
-				waiter->mapname, waiter->event_nr);
-
-		/*
-		 * event might be :
-		 *
-		 * 1) a table reload, which means our mpp structure is
-		 *    obsolete : refresh it through update_multipath()
-		 * 2) a path failed by DM : mark as such through
-		 *    update_multipath()
-		 * 3) map has gone away : stop the thread.
-		 * 4) a path reinstate : nothing to do
-		 * 5) a switch group : nothing to do
-		 */
-		pthread_cleanup_push(cleanup_lock, waiter->vecs->lock);
-		lock(waiter->vecs->lock);
-		r = update_multipath(waiter->vecs, waiter->mapname);
-		lock_cleanup_pop(waiter->vecs->lock);
-
-		if (r)
-			return -1; /* stop the thread */
-
-		event_nr = dm_geteventnr(waiter->mapname);
-
-		if (waiter->event_nr == event_nr)
-			return 1; /* upon problem reschedule 1s later */
-
-		waiter->event_nr = event_nr;
-	}
-	return -1; /* never reach there */
-}
-
-static void *
-waitevent (void * et)
-{
-	int r;
-	struct event_thread *waiter;
-
-	mlockall(MCL_CURRENT | MCL_FUTURE);
-
-	waiter = (struct event_thread *)et;
-	pthread_cleanup_push(free_waiter, et);
-
-	while (1) {
-		r = waiteventloop(waiter);
-
-		if (r < 0)
-			break;
-
-		sleep(r);
-	}
-
-	pthread_cleanup_pop(1);
-	return NULL;
-}
-
-static int
-start_waiter_thread (struct multipath * mpp, struct vectors * vecs)
-{
-	pthread_attr_t attr;
-	struct event_thread * wp;
-
-	if (!mpp)
-		return 0;
-
-	if (pthread_attr_init(&attr))
-		goto out;
-
-	pthread_attr_setstacksize(&attr, 32 * 1024);
-	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-
-	wp = alloc_waiter();
-
-	if (!wp)
-		goto out;
-
-	mpp->waiter = (void *)wp;
-	strncpy(wp->mapname, mpp->alias, WWID_SIZE);
-	wp->vecs = vecs;
-	wp->mpp = mpp;
-
-	if (pthread_create(&wp->thread, &attr, waitevent, wp)) {
-		condlog(0, "%s: cannot create event checker", wp->mapname);
-		goto out1;
-	}
-	condlog(2, "%s: event checker started", wp->mapname);
-
-	return 0;
-out1:
-	free_waiter(wp);
-	mpp->waiter = NULL;
-out:
-	condlog(0, "failed to start waiter thread");
-	return 1;
-}
-
 static void
 sync_map_state(struct multipath *mpp)
 {
-	int i, j;
 	struct pathgroup *pgp;
-        struct path *pp;
+	struct path *pp;
+	unsigned int i, j;
 
 	vector_foreach_slot (mpp->pg, pgp, i){
 		vector_foreach_slot (pgp->paths, pp, j){
@@ -502,10 +169,10 @@ sync_map_state(struct multipath *mpp)
 static void
 sync_maps_state(vector mpvec)
 {
-	int i;
+	unsigned int i;
 	struct multipath *mpp;
 
-	vector_foreach_slot (mpvec, mpp, i) 
+	vector_foreach_slot (mpvec, mpp, i)
 		sync_map_state(mpp);
 }
 
@@ -536,52 +203,50 @@ flush_map(struct multipath * mpp, struct vectors * vecs)
 }
 
 static int
-uev_add_map (char * devname, struct vectors * vecs)
+uev_add_map (struct sysfs_device * dev, struct vectors * vecs)
 {
-	condlog(2, "%s: add map (uevent)", devname);
-	return ev_add_map(devname, vecs);
+	condlog(2, "%s: add map (uevent)", dev->kernel);
+	return ev_add_map(dev, vecs);
 }
 
 int
-ev_add_map (char * devname, struct vectors * vecs)
+ev_add_map (struct sysfs_device * dev, struct vectors * vecs)
 {
-	int major, minor;
-	char dev_t[BLK_DEV_SIZE];
 	char * alias;
+	char *dev_t;
+	int major, minor;
 	char * refwwid;
 	struct multipath * mpp;
 	int map_present;
 	int r = 1;
 
-	if (sscanf(devname, "dm-%d", &minor) == 1 &&
-	    !sysfs_get_dev(sysfs_path, devname, dev_t, BLK_DEV_SIZE) &&
-	    sscanf(dev_t, "%d:%d", &major, &minor) == 2)
-		alias = dm_mapname(major, minor);
-	else
-		alias = STRDUP(devname);
-		
+	dev_t = sysfs_attr_get_value(dev->devpath, "dev");
+
+	if (!dev_t || sscanf(dev_t, "%d:%d", &major, &minor) != 2)
+		return 1;
+
+	alias = dm_mapname(major, minor);
+
 	if (!alias)
 		return 1;
-	
+
 	map_present = dm_map_present(alias);
 
 	if (map_present && dm_type(alias, DEFAULT_TARGET) <= 0) {
 		condlog(4, "%s: not a multipath map", alias);
-		FREE(alias);
 		return 0;
 	}
 
 	mpp = find_mp_by_alias(vecs->mpvec, alias);
 
 	if (mpp) {
-                /*
+		/*
 		 * Not really an error -- we generate our own uevent
 		 * if we create a multipath mapped device as a result
 		 * of uev_add_path
 		 */
 		condlog(0, "%s: devmap already registered",
-			devname);
-		FREE(alias);
+			dev->kernel);
 		return 0;
 	}
 
@@ -591,31 +256,30 @@ ev_add_map (char * devname, struct vectors * vecs)
 	if (map_present && (mpp = add_map_without_path(vecs, minor, alias,
 					start_waiter_thread))) {
 		sync_map_state(mpp);
-		condlog(3, "%s: devmap %s added", alias, devname);
+		condlog(3, "%s: devmap %s added", alias, dev->kernel);
 		return 0;
 	}
-	refwwid = get_refwwid(devname, DEV_DEVMAP, vecs->pathvec);
+	refwwid = get_refwwid(dev->kernel, DEV_DEVMAP, vecs->pathvec);
 
 	if (refwwid) {
 		r = coalesce_paths(vecs, NULL, refwwid);
 		dm_lib_release();
 	}
-	
+
 	if (!r)
-		condlog(3, "%s: devmap %s added", alias, devname);
+		condlog(3, "%s: devmap %s added", alias, dev->kernel);
 	else
-		condlog(0, "%s: uev_add_map %s failed", alias, devname);
+		condlog(0, "%s: uev_add_map %s failed", alias, dev->kernel);
 
 	FREE(refwwid);
-	FREE(alias);
 	return r;
 }
 
 static int
-uev_remove_map (char * devname, struct vectors * vecs)
+uev_remove_map (struct sysfs_device * dev, struct vectors * vecs)
 {
-	condlog(2, "%s: remove map (uevent)", devname);
-	return ev_remove_map(devname, vecs);
+	condlog(2, "%s: remove map (uevent)", dev->kernel);
+	return ev_remove_map(dev->kernel, vecs);
 }
 
 int
@@ -636,13 +300,13 @@ ev_remove_map (char * devname, struct vectors * vecs)
 }
 
 static int
-uev_umount_map (char * devname, struct vectors * vecs)
+uev_umount_map (struct sysfs_device * dev, struct vectors * vecs)
 {
 	struct multipath * mpp;
 
-	condlog(2, "%s: umount map (uevent)", devname);
+	condlog(2, "%s: umount map (uevent)", dev->kernel);
 
-	mpp = find_mp_by_str(vecs->mpvec, devname);
+	mpp = find_mp_by_str(vecs->mpvec, dev->kernel);
 
 	if (!mpp)
 		return 0;
@@ -655,12 +319,12 @@ uev_umount_map (char * devname, struct vectors * vecs)
 
 	return 0;
 }
-	
+
 static int
-uev_add_path (char * devname, struct vectors * vecs)
+uev_add_path (struct sysfs_device * dev, struct vectors * vecs)
 {
-	condlog(2, "%s: add path (uevent)", devname);
-	return (ev_add_path(devname, vecs) != 1)? 0 : 1;
+	condlog(2, "%s: add path (uevent)", dev->kernel);
+	return (ev_add_path(dev->kernel, vecs) != 1)? 0 : 1;
 }
 
 
@@ -704,13 +368,13 @@ ev_add_path (char * devname, struct vectors * vecs)
 		condlog(0, "%s: failed to get path uid", devname);
 		return 1; /* leave path added to pathvec */
 	}
-	if (blacklist_path(conf, pp)){
+	if (filter_path(conf, pp)){
 		int i = find_slot(vecs->pathvec, (void *)pp);
 		if (i != -1)
 			vector_del_slot(vecs->pathvec, i);
 		free_path(pp);
 		return 2;
-	}	
+	}
 	mpp = pp->mpp = find_mp_by_wwid(vecs->mpvec, pp->wwid);
 rescan:
 	if (mpp) {
@@ -742,8 +406,8 @@ rescan:
 		condlog(0, "%s: failed in domap for addition of new "
 			"path %s", mpp->alias, devname);
 		/*
- 		 * deal with asynchronous uevents :((
- 		 */
+		 * deal with asynchronous uevents :((
+		 */
 		if (mpp->action == ACT_RELOAD) {
 			condlog(0, "%s: uev_add_path sleep", mpp->alias);
 			sleep(1);
@@ -776,10 +440,16 @@ out:
 }
 
 static int
-uev_remove_path (char * devname, struct vectors * vecs)
+uev_remove_path (struct sysfs_device * dev, struct vectors * vecs)
 {
-	condlog(2, "%s: remove path (uevent)", devname);
-	return ev_remove_path(devname, vecs);
+	int retval;
+
+	condlog(2, "%s: remove path (uevent)", dev->kernel);
+	retval = ev_remove_path(dev->kernel, vecs);
+	if (!retval)
+		sysfs_device_put(dev);
+
+	return retval;
 }
 
 int
@@ -809,9 +479,9 @@ ev_remove_path (char * devname, struct vectors * vecs)
 			vector rpvec = vector_alloc();
 
 			/*
-	 	 	 * transform the mp->pg vector of vectors of paths
-	 	 	 * into a mp->params string to feed the device-mapper
-	 	 	 */
+			 * transform the mp->pg vector of vectors of paths
+			 * into a mp->params string to feed the device-mapper
+			 */
 			update_mpp_paths(mpp, vecs->pathvec);
 			if ((i = find_slot(mpp->paths, (void *)pp)) != -1)
 				vector_del_slot(mpp->paths, i);
@@ -837,8 +507,8 @@ ev_remove_path (char * devname, struct vectors * vecs)
 					goto out;
 				}
 				/*
-	 	 	 	 * reload the map
-	 	 	 	 */
+				 * reload the map
+				 */
 				mpp->action = ACT_RELOAD;
 				if (domap(mpp) <= 0) {
 					condlog(0, "%s: failed in domap for "
@@ -863,7 +533,7 @@ ev_remove_path (char * devname, struct vectors * vecs)
 				}
 				sync_map_state(mpp);
 
-				condlog(3, "%s path removed from devmap %s",
+				condlog(3, "%s: path removed from map %s",
 					devname, mpp->alias);
 			}
 			free_pathvec(rpvec, KEEP_PATHS);
@@ -898,8 +568,8 @@ out:
 static int
 map_discovery (struct vectors * vecs)
 {
-	int i;
 	struct multipath * mpp;
+	unsigned int i;
 
 	if (dm_get_maps(vecs->mpvec, "multipath"))
 		return 1;
@@ -916,7 +586,7 @@ uxsock_trigger (char * str, char ** reply, int * len, void * trigger_data)
 {
 	struct vectors * vecs;
 	int r;
-	
+
 	*reply = NULL;
 	*len = 0;
 	vecs = (struct vectors *)trigger_data;
@@ -958,11 +628,11 @@ uev_discard(char * devpath)
 	return 0;
 }
 
-int 
+int
 uev_trigger (struct uevent * uev, void * trigger_data)
 {
 	int r = 0;
-	char devname[32];
+	struct sysfs_device *sysdev;
 	struct vectors * vecs;
 
 	vecs = (struct vectors *)trigger_data;
@@ -970,40 +640,46 @@ uev_trigger (struct uevent * uev, void * trigger_data)
 	if (uev_discard(uev->devpath))
 		return 0;
 
-	basename(uev->devpath, devname);
+	sysdev = sysfs_device_get(uev->devpath);
+	if(!sysdev)
+		return 0;
+
 	lock(vecs->lock);
 
 	/*
-	 * device map add/remove event
+	 * device map event
+	 * Add events are ignored here as the tables
+	 * are not fully initialised then.
 	 */
-	if (!strncmp(devname, "dm-", 3)) {
-		if (!strncmp(uev->action, "add", 3)) {
-			r = uev_add_map(devname, vecs);
+	if (!strncmp(sysdev->kernel, "dm-", 3)) {
+		if (!strncmp(uev->action, "change", 6)) {
+			r = uev_add_map(sysdev, vecs);
 			goto out;
 		}
 		if (!strncmp(uev->action, "remove", 6)) {
-			r = uev_remove_map(devname, vecs);
+			r = uev_remove_map(sysdev, vecs);
 			goto out;
 		}
 		if (!strncmp(uev->action, "umount", 6)) {
-			r = uev_umount_map(devname, vecs);
+			r = uev_umount_map(sysdev, vecs);
 			goto out;
 		}
 		goto out;
 	}
-	
+
 	/*
 	 * path add/remove event
 	 */
-	if (blacklist(conf->blist_devnode, devname))
+	if (filter_devnode(conf->blist_devnode, conf->elist_devnode,
+			   sysdev->kernel) > 0)
 		goto out;
 
 	if (!strncmp(uev->action, "add", 3)) {
-		r = uev_add_path(devname, vecs);
+		r = uev_add_path(sysdev, vecs);
 		goto out;
 	}
 	if (!strncmp(uev->action, "remove", 6)) {
-		r = uev_remove_path(devname, vecs);
+		r = uev_remove_path(sysdev, vecs);
 		goto out;
 	}
 
@@ -1017,37 +693,36 @@ ueventloop (void * ap)
 {
 	if (uevent_listen(&uev_trigger, ap))
 		fprintf(stderr, "error starting uevent listener");
-		
+
 	return NULL;
 }
 
 static void *
 uxlsnrloop (void * ap)
 {
-	if (load_keys())
-		return NULL;
-	
-	if (alloc_handlers())
+	if (cli_init())
 		return NULL;
 
-	add_handler(LIST+PATHS, cli_list_paths);
-	add_handler(LIST+MAPS, cli_list_maps);
-	add_handler(LIST+MAPS+STATUS, cli_list_maps_status);
-	add_handler(LIST+MAPS+STATS, cli_list_maps_stats);
-	add_handler(LIST+MAPS+TOPOLOGY, cli_list_maps_topology);
-	add_handler(LIST+TOPOLOGY, cli_list_maps_topology);
-	add_handler(LIST+MAP+TOPOLOGY, cli_list_map_topology);
-	add_handler(LIST+CONFIG, cli_list_config);
-	add_handler(ADD+PATH, cli_add_path);
-	add_handler(DEL+PATH, cli_del_path);
-	add_handler(ADD+MAP, cli_add_map);
-	add_handler(DEL+MAP, cli_del_map);
-	add_handler(SWITCH+MAP+GROUP, cli_switch_group);
-	add_handler(RECONFIGURE, cli_reconfigure);
-	add_handler(SUSPEND+MAP, cli_suspend);
-	add_handler(RESUME+MAP, cli_resume);
-	add_handler(REINSTATE+PATH, cli_reinstate);
-	add_handler(FAIL+PATH, cli_fail);
+	set_handler_callback(LIST+PATHS, cli_list_paths);
+	set_handler_callback(LIST+MAPS, cli_list_maps);
+	set_handler_callback(LIST+MAPS+STATUS, cli_list_maps_status);
+	set_handler_callback(LIST+MAPS+STATS, cli_list_maps_stats);
+	set_handler_callback(LIST+MAPS+TOPOLOGY, cli_list_maps_topology);
+	set_handler_callback(LIST+TOPOLOGY, cli_list_maps_topology);
+	set_handler_callback(LIST+MAP+TOPOLOGY, cli_list_map_topology);
+	set_handler_callback(LIST+CONFIG, cli_list_config);
+	set_handler_callback(LIST+BLACKLIST, cli_list_blacklist);
+	set_handler_callback(LIST+DEVICES, cli_list_devices);
+	set_handler_callback(ADD+PATH, cli_add_path);
+	set_handler_callback(DEL+PATH, cli_del_path);
+	set_handler_callback(ADD+MAP, cli_add_map);
+	set_handler_callback(DEL+MAP, cli_del_map);
+	set_handler_callback(SWITCH+MAP+GROUP, cli_switch_group);
+	set_handler_callback(RECONFIGURE, cli_reconfigure);
+	set_handler_callback(SUSPEND+MAP, cli_suspend);
+	set_handler_callback(RESUME+MAP, cli_resume);
+	set_handler_callback(REINSTATE+PATH, cli_reinstate);
+	set_handler_callback(FAIL+PATH, cli_fail);
 
 	uxsock_listen(&uxsock_trigger, ap);
 
@@ -1113,12 +788,12 @@ enable_group(struct path * pp)
 	 *
 	 * we can safely return here, because upon map reload, all
 	 * PG will be enabled.
- 	 */
+	 */
 	if (!pp->mpp->pg || !pp->pgindex)
 		return;
 
 	pgp = VECTOR_SLOT(pp->mpp->pg, pp->pgindex - 1);
-	
+
 	if (pgp->status == PGSTATE_DISABLED) {
 		condlog(2, "%s: enable group #%i", pp->mpp->alias, pp->pgindex);
 		dm_enablegroup(pp->mpp->alias, pp->pgindex);
@@ -1129,7 +804,7 @@ static void
 mpvec_garbage_collector (struct vectors * vecs)
 {
 	struct multipath * mpp;
-	int i;
+	unsigned int i;
 
 	vector_foreach_slot (vecs->mpvec, mpp, i) {
 		if (mpp && mpp->alias && !dm_map_present(mpp->alias)) {
@@ -1144,7 +819,7 @@ static void
 defered_failback_tick (vector mpvec)
 {
 	struct multipath * mpp;
-	int i;
+	unsigned int i;
 
 	vector_foreach_slot (mpvec, mpp, i) {
 		/*
@@ -1163,7 +838,7 @@ static void
 retry_count_tick(vector mpvec)
 {
 	struct multipath *mpp;
-	int i;
+	unsigned int i;
 
 	vector_foreach_slot (mpvec, mpp, i) {
 		if (mpp->retry_tick) {
@@ -1182,8 +857,9 @@ checkerloop (void *ap)
 {
 	struct vectors *vecs;
 	struct path *pp;
-	int i, count = 0;
+	int count = 0;
 	int newstate;
+	unsigned int i;
 
 	mlockall(MCL_CURRENT | MCL_FUTURE);
 	vecs = (struct vectors *)ap;
@@ -1213,24 +889,36 @@ checkerloop (void *ap)
 			 * in case we exit abnormaly from here
 			 */
 			pp->tick = conf->checkint;
-			
+
 			if (!checker_selected(&pp->checker)) {
 				pathinfo(pp, conf->hwtable, DI_SYSFS);
 				select_checker(pp);
 			}
-
 			if (!checker_selected(&pp->checker)) {
 				condlog(0, "%s: checker is not set", pp->dev);
 				continue;
 			}
+			/*
+			 * Set checker in async mode.
+			 * Honored only by checker implementing the said mode.
+			 */
+			checker_set_async(&pp->checker);
+
 			newstate = checker_check(&pp->checker);
-			
+
 			if (newstate < 0) {
 				condlog(2, "%s: unusable path", pp->dev);
 				pathinfo(pp, conf->hwtable, 0);
 				continue;
 			}
-
+			/*
+			 * Async IO in flight. Keep the previous path state
+			 * and reschedule as soon as possible
+			 */
+			if (newstate == PATH_PENDING) {
+				pp->tick = 1;
+				continue;
+			}
 			if (newstate != pp->state) {
 				int oldstate = pp->state;
 				pp->state = newstate;
@@ -1245,7 +933,7 @@ checkerloop (void *ap)
 				if (newstate == PATH_DOWN ||
 				    newstate == PATH_SHAKY ||
 				    update_multipath_strings(pp->mpp,
-					    		     vecs->pathvec)) {
+							     vecs->pathvec)) {
 					/*
 					 * proactively fail path in the DM
 					 */
@@ -1336,7 +1024,7 @@ checkerloop (void *ap)
 			mpvec_garbage_collector(vecs);
 			count = MAPGCINT;
 		}
-		
+
 		lock_cleanup_pop(vecs->lock);
 		sleep(1);
 	}
@@ -1351,12 +1039,12 @@ configure (struct vectors * vecs, int start_waiters)
 	vector mpvec;
 	int i;
 
-	if (!(vecs->pathvec = vector_alloc()))
+	if (!vecs->pathvec && !(vecs->pathvec = vector_alloc()))
 		return 1;
-	
-	if (!(vecs->mpvec = vector_alloc()))
+
+	if (!vecs->mpvec && !(vecs->mpvec = vector_alloc()))
 		return 1;
-	
+
 	if (!(mpvec = vector_alloc()))
 		return 1;
 
@@ -1366,11 +1054,11 @@ configure (struct vectors * vecs, int start_waiters)
 	path_discovery(vecs->pathvec, conf, DI_ALL);
 
 	vector_foreach_slot (vecs->pathvec, pp, i){
-		if (blacklist_path(conf, pp)){
+		if (filter_path(conf, pp)){
 			vector_del_slot(vecs->pathvec, i);
 			free_path(pp);
 			i--;
-		}	
+		}
 		else
 			pp->checkint = conf->checkint;
 	}
@@ -1394,10 +1082,6 @@ configure (struct vectors * vecs, int start_waiters)
 
 	sync_maps_state(mpvec);
 
-	if (conf->verbosity > 2)
-		vector_foreach_slot(mpvec, mpp, i)
-			print_map(mpp);
-
 	/*
 	 * purge dm of old maps
 	 */
@@ -1406,6 +1090,7 @@ configure (struct vectors * vecs, int start_waiters)
 	/*
 	 * save new set of maps formed by considering current path state
 	 */
+	vector_free(vecs->mpvec);
 	vecs->mpvec = mpvec;
 
 	/*
@@ -1435,6 +1120,7 @@ reconfigure (struct vectors * vecs)
 	if (VECTOR_SIZE(vecs->pathvec))
 		free_pathvec(vecs->pathvec, FREE_PATHS);
 
+	vecs->pathvec = NULL;
 	conf = NULL;
 
 	if (load_config(DEFAULT_CONFIGFILE))
@@ -1461,30 +1147,16 @@ init_vecs (void)
 	if (!vecs)
 		return NULL;
 
-	vecs->lock = 
+	vecs->lock =
 		(pthread_mutex_t *)MALLOC(sizeof(pthread_mutex_t));
 
 	if (!vecs->lock)
 		goto out;
 
-	vecs->pathvec = vector_alloc();
-
-	if (!vecs->pathvec)
-		goto out1;
-		
-	vecs->mpvec = vector_alloc();
-
-	if (!vecs->mpvec)
-		goto out2;
-	
 	pthread_mutex_init(vecs->lock, NULL);
 
 	return vecs;
 
-out2:
-	vector_free(vecs->pathvec);
-out1:
-	FREE(vecs->lock);
 out:
 	FREE(vecs);
 	condlog(0, "failed to init paths");
@@ -1543,21 +1215,21 @@ signal_init(void)
 	signal_set(SIGUSR1, sigusr1);
 	signal_set(SIGINT, sigend);
 	signal_set(SIGTERM, sigend);
-	signal_set(SIGKILL, sigend);
+	signal(SIGPIPE, SIG_IGN);
 }
 
 static void
 setscheduler (void)
 {
-        int res;
+	int res;
 	static struct sched_param sched_param = {
-		sched_priority: 99
+		.sched_priority = 99
 	};
 
-        res = sched_setscheduler (0, SCHED_RR, &sched_param);
+	res = sched_setscheduler (0, SCHED_RR, &sched_param);
 
-        if (res == -1)
-                condlog(LOG_WARNING, "Could not set SCHED_RR at priority 99");
+	if (res == -1)
+		condlog(LOG_WARNING, "Could not set SCHED_RR at priority 99");
 	return;
 }
 
@@ -1574,7 +1246,7 @@ set_oom_adj (int val)
 	fprintf(fp, "%i", val);
 	fclose(fp);
 }
-	
+
 static int
 child (void * param)
 {
@@ -1617,7 +1289,7 @@ child (void * param)
 	if (!vecs)
 		exit(1);
 
-	if (sysfs_get_mnt_path(sysfs_path, FILE_NAME_SIZE)) {
+	if (sysfs_init(conf->sysfs_dir, FILE_NAME_SIZE)) {
 		condlog(0, "can not find sysfs mount point");
 		exit(1);
 	}
@@ -1636,7 +1308,7 @@ child (void * param)
 	pthread_attr_init(&attr);
 	pthread_attr_setstacksize(&attr, 64 * 1024);
 	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-	
+
 	pthread_create(&check_thr, &attr, checkerloop, vecs);
 	pthread_create(&uevent_thr, &attr, ueventloop, vecs);
 	pthread_create(&uxlsnr_thr, &attr, uxlsnrloop, vecs);
@@ -1654,6 +1326,8 @@ child (void * param)
 	pthread_cancel(uevent_thr);
 	pthread_cancel(uxlsnr_thr);
 
+	sysfs_cleanup();
+
 	free_keys(keys);
 	keys = NULL;
 	free_handlers(handlers);
@@ -1670,7 +1344,7 @@ child (void * param)
 	conf = NULL;
 
 	condlog(2, "--------shut down-------");
-	
+
 	if (logsink)
 		log_thread_stop();
 
@@ -1738,8 +1412,9 @@ main (int argc, char *argv[])
 	extern int optind;
 	int arg;
 	int err;
-	
+
 	logsink = 1;
+	dm_init();
 
 	if (getuid() != 0) {
 		fprintf(stderr, "need to be root\n");
@@ -1780,7 +1455,7 @@ main (int argc, char *argv[])
 		err = 0;
 	else
 		err = daemonize();
-	
+
 	if (err < 0)
 		/* error */
 		exit(1);
